# -*- coding: utf-8 -*-

"""
@author: <sylvain.herledan@oceandatalab.com>
@date: 2016-11-02

Copyright (C) 2016-2019 OceanDataLab

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as
published by the Free Software Foundation, either version 3 of the
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

import logging
import flatbuffers
import SEAScope.API.OpCode
import SEAScope.API.CommandArgs
import SEAScope.API.AddGranule
import SEAScope.API.Command
import SEAScope.types.granule_metadata
import SEAScope.types.granule_data
import SEAScope.types.renderable_id

logger = logging.getLogger(__name__)


def serialize(builder, cmd_obj):
    """"""
    if builder is None:
        builder = flatbuffers.Builder(0)

    serializer = SEAScope.types.granule_data.serialize
    _, data = serializer(builder, cmd_obj['data'])
    serializer = SEAScope.types.granule_metadata.serialize
    _, metadata = serializer(builder, cmd_obj['metadata'])
    serializer = SEAScope.types.renderable_id.serialize
    _, rid = serializer(builder, cmd_obj['id'])

    cls = SEAScope.API.AddGranule
    cls.AddGranuleStart(builder)
    cls.AddGranuleAddId(builder, rid)
    cls.AddGranuleAddMetadata(builder, metadata)
    cls.AddGranuleAddData(builder, data)
    cmd_args = cls.AddGranuleEnd(builder)

    opcode = SEAScope.API.OpCode.OpCode().addGranule
    args_type = SEAScope.API.CommandArgs.CommandArgs().AddGranule
    SEAScope.API.Command.CommandStart(builder)
    SEAScope.API.Command.CommandAddOpcode(builder, opcode)
    SEAScope.API.Command.CommandAddArgsType(builder, args_type)
    SEAScope.API.Command.CommandAddArgs(builder, cmd_args)
    cmd = SEAScope.API.Command.CommandEnd(builder)

    return builder, cmd
