# -*- coding: utf-8 -*-

"""
@author: <sylvain.herledan@oceandatalab.com>
@date: 2016-09-08

Copyright (C) 2016-2019 OceanDataLab

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as
published by the Free Software Foundation, either version 3 of the
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

import logging
import flatbuffers
import SEAScope.API.OpCode
import SEAScope.API.SearchGranules
import SEAScope.API.SearchGranulesResponse
import SEAScope.API.Command
import SEAScope.types.collection_id
import SEAScope.types.granule_metadata

logger = logging.getLogger(__name__)


def serialize(builder, search_obj):
    """"""
    if builder is None:
        builder = flatbuffers.Builder(0)

    cls = SEAScope.API.SearchGranules

    collectionsOffsets = 0
    cls.SearchGranulesStartCollectionsVector(builder,
                                             len(search_obj['collections']))
    for collection in search_obj['collections']:
        SEAScope.types.collection_id.serialize(builder, collection)
    collectionsOffsets = builder.EndVector(len(search_obj['collections']))

    cls.SearchGranulesStart(builder)
    cls.SearchGranulesAddCurrent(builder, search_obj['current'])
    cls.SearchGranulesAddStart(builder, search_obj['start'])
    cls.SearchGranulesAddStop(builder, search_obj['stop'])
    cls.SearchGranulesAddZoom(builder, search_obj['zoom'])
    cls.SearchGranulesAddCollections(builder, collectionsOffsets)
    cmd_args = cls.SearchGranulesEnd(builder)

    opcode = SEAScope.API.OpCode.OpCode().searchGranules
    args_type = SEAScope.API.CommandArgs.CommandArgs().SearchGranules
    SEAScope.API.Command.CommandStart(builder)
    SEAScope.API.Command.CommandAddOpcode(builder, opcode)
    SEAScope.API.Command.CommandAddArgsType(builder, args_type)
    SEAScope.API.Command.CommandAddArgs(builder, cmd_args)
    cmd = SEAScope.API.Command.CommandEnd(builder)

    return builder, cmd


def deserialize(buf):
    """"""
    deserializer = SEAScope.types.granule_metadata.deserialize
    cls = SEAScope.API.SearchGranulesResponse.SearchGranulesResponse
    res = cls.GetRootAsSearchGranulesResponse(buf, 0)

    granules_len = res.GranulesLength()
    return [deserializer(res.Granules(x)) for x in range(0, granules_len)]
