# -*- coding: utf-8 -*-

"""
@author: <sylvain.herledan@oceandatalab.com>
@date: 2016-09-07

Copyright (C) 2016-2019 OceanDataLab

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as
published by the Free Software Foundation, either version 3 of the
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

import logging
import flatbuffers
import SEAScope.API.Collection
import SEAScope.types.renderable_id
import SEAScope.types.tag
import SEAScope.types.variable

logger = logging.getLogger(__name__)


def serialize(builder, collection_obj):
    """"""
    if builder is None:
        builder = flatbuffers.Builder(0)

    cls = SEAScope.API.Collection

    _must_be_current = collection_obj['mustBeCurrent']
    _x_seamless = collection_obj['xSeamless']
    _y_seamless = collection_obj['ySeamless']
    _news_aligned = collection_obj['NEWSAligned']
    _label = builder.CreateString(collection_obj['label'])

    tags = []
    tags_len = len(collection_obj['tags'].keys())
    for k, v in collection_obj['tags'].items():
        _, tag = SEAScope.types.tag.serialize(builder, k, v)
        tags.append(tag)
    cls.CollectionStartTagsVector(builder, tags_len)
    for t in tags:
        builder.PrependUOffsetTRelative(t)
    _tags = builder.EndVector(tags_len)

    variables = []
    vars_len = len(collection_obj['variables'])
    for var_obj in collection_obj['variables']:
        _, variable = SEAScope.types.variable.serialize(builder, var_obj)
        variables.append(variable)
    cls.CollectionStartVariablesVector(builder, vars_len)
    for v in variables:
        builder.PrependUOffsetTRelative(v)
    _variables = builder.EndVector(vars_len)

    _default_variable = int(collection_obj['defaultVariable'])
    _, _rid = SEAScope.types.renderable_id.serialize(builder,
                                                     collection_obj['id'])

    cls.CollectionStart(builder)
    cls.CollectionAddId(builder, _rid)
    cls.CollectionAddMustBeCurrent(builder, _must_be_current)
    cls.CollectionAddXSeamless(builder, _x_seamless)
    cls.CollectionAddYSeamless(builder, _y_seamless)
    cls.CollectionAddNewsAligned(builder, _news_aligned)
    cls.CollectionAddLabel(builder, _label)
    cls.CollectionAddTags(builder, _tags)
    cls.CollectionAddVariables(builder, _variables)
    cls.CollectionAddDefaultVariable(builder, _default_variable)
    collection = cls.CollectionEnd(builder)

    return builder, collection


def deserialize(buf):
    """"""
    col_obj = {}
    col_obj['id'] = SEAScope.types.renderable_id.deserialize(buf.Id())
    col_obj['label'] = buf.Label()
    col_obj['mustBeCurrent'] = 0 < buf.MustBeCurrent()
    col_obj['xSeamless'] = 0 < buf.XSeamless()
    col_obj['ySeamless'] = 0 < buf.YSeamless()
    col_obj['NEWSAligned'] = 0 < buf.NewsAligned()
    col_obj['tags'] = {}
    tags_count = buf.TagsLength()
    for i in range(0, tags_count):
        name, value = SEAScope.types.tag.deserialize(buf.Tags(i))
        col_obj['tags'][name] = value

    col_obj['variables'] = []
    vars_count = buf.VariablesLength()
    for i in range(0, vars_count):
        v = SEAScope.types.variable.deserialize(buf.Variables(i))
        col_obj['variables'].append(v)

    return col_obj
