# -*- coding: utf-8 -*-

"""
@author: <sylvain.herledan@oceandatalab.com>
@date: 2016-11-02

Copyright (C) 2016-2019 OceanDataLab

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as
published by the Free Software Foundation, either version 3 of the
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

import logging
import flatbuffers
import SEAScope.API.DataInfo
import SEAScope.API.DataType

logger = logging.getLogger(__name__)
data_types = {
    'none': SEAScope.API.DataType.DataType.NONE,
    'uint8': SEAScope.API.DataType.DataType.UBYTE,
    'ushort16': SEAScope.API.DataType.DataType.USHORT,
    'uint32': SEAScope.API.DataType.DataType.UINT,
    'float32': SEAScope.API.DataType.DataType.FLOAT32}


def serialize(builder, info_obj):
    """"""
    if builder is None:
        builder = flatbuffers.Builder(0)

    cls = SEAScope.API.DataInfo

    offsets_len = len(info_obj['offsets'])
    cls.DataInfoStartOffsetsVector(builder, offsets_len)
    for v in info_obj['offsets'][::-1]:
        builder.PrependFloat32(v)
    offsets = builder.EndVector(offsets_len)

    scales_len = len(info_obj['scaleFactors'])
    cls.DataInfoStartScaleFactorsVector(builder, scales_len)
    for v in info_obj['scaleFactors'][::-1]:
        builder.PrependFloat32(v)
    scale_factors = builder.EndVector(scales_len)

    fills_len = len(info_obj['fillValues'])
    cls.DataInfoStartFillValuesVector(builder, fills_len)
    for v in info_obj['fillValues'][::-1]:
        builder.PrependFloat32(v)
    fill_values = builder.EndVector(fills_len)

    _data_type = data_types.get(info_obj['dataType'], data_types['none'])
    print(_data_type)

    cls.DataInfoStart(builder)
    cls.DataInfoAddChannels(builder, int(info_obj['channels']))
    cls.DataInfoAddXArity(builder, int(info_obj['xArity']))
    cls.DataInfoAddYArity(builder, int(info_obj['yArity']))
    cls.DataInfoAddDataType(builder, _data_type)
    cls.DataInfoAddOffsets(builder, offsets)
    cls.DataInfoAddScaleFactors(builder, scale_factors)
    cls.DataInfoAddFillValues(builder, fill_values)
    data_info = cls.DataInfoEnd(builder)

    return builder, data_info


def deserialize(o):
    """"""
    pass
