# -*- coding: utf-8 -*-

"""
@author: <sylvain.herledan@oceandatalab.com>
@date: 2016-10-11

Copyright (C) 2016-2019 OceanDataLab

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as
published by the Free Software Foundation, either version 3 of the
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

import logging
import flatbuffers
import SEAScope.API.IDFDescriptor
import SEAScope.types.shape

logger = logging.getLogger(__name__)


def serialize(builder, obj):
    """"""
    if builder is None:
        builder = flatbuffers.Builder(0)

    uri = builder.CreateString(obj['uri'])
    resolution = obj['resolution']
    subsampling_factor = int(obj['subsampling_factor'])
    x_arity = int(obj['xArity'])
    y_arity = int(obj['yArity'])
    _, shape = SEAScope.types.shape.serialize(builder, obj['shape'])

    cls = SEAScope.API.IDFDescriptor
    cls.IDFDescriptorStart(builder)
    cls.IDFDescriptorAddUri(builder, uri)
    cls.IDFDescriptorAddResolution(builder, resolution)
    cls.IDFDescriptorAddXArity(builder, x_arity)
    cls.IDFDescriptorAddYArity(builder, y_arity)
    cls.IDFDescriptorAddShape(builder, shape)
    cls.IDFDescriptorAddSubsamplingFactor(builder, subsampling_factor)
    idf_descriptor = cls.IDFDescriptorEnd(builder)

    return builder, idf_descriptor


def deserialize(o):
    """"""
    result = {}
    result['uri'] = o.Uri()
    result['resolution'] = o.Resolution()
    result['xArity'] = o.XArity()
    result['yArity'] = o.YArity()
    result['shape'] = SEAScope.types.shape.deserialize(o.Shape())
    result['subsampling_factor'] = o.SubsamplingFactor()
    return result
