# -*- coding: utf-8 -*-

"""
@author: <sylvain.herledan@oceandatalab.com>
@date: 2016-09-09

Copyright (C) 2016-2019 OceanDataLab

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as
published by the Free Software Foundation, either version 3 of the
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

import logging
import flatbuffers
import SEAScope.API.RenderingCfg
import SEAScope.types.color
import SEAScope.types.renderable_id
import SEAScope.API.FilterMode

logger = logging.getLogger(__name__)

rendering_methods = {
    'RASTER': SEAScope.API.RenderingMethod.RenderingMethod.RASTER,
    'ARROWS': SEAScope.API.RenderingMethod.RenderingMethod.ARROWS,
    'BARBS': SEAScope.API.RenderingMethod.RenderingMethod.BARBS,
    'TRAJECTORIES': SEAScope.API.RenderingMethod.RenderingMethod.TRAJECTORIES,
    'DOTSCLOUD': SEAScope.API.RenderingMethod.RenderingMethod.DOTSCLOUD,
    'STREAMLINES': SEAScope.API.RenderingMethod.RenderingMethod.STREAMLINES,
    'RAWRGB': SEAScope.API.RenderingMethod.RenderingMethod.RAWRGB}

filter_modes = {
    'NEAREST': SEAScope.API.FilterMode.FilterMode.NEAREST,
    'BILINEAR': SEAScope.API.FilterMode.FilterMode.BILINEAR}


def serialize(builder, cfg_obj):
    """"""
    if builder is None:
        builder = flatbuffers.Builder(0)

    cls = SEAScope.API.RenderingCfg

    _rendered = cfg_obj['rendered']
    _logscale = cfg_obj['logscale']
    _min = cfg_obj['min']
    _max = cfg_obj['max']
    _opacity = cfg_obj['opacity']
    _zindex = cfg_obj['zindex']
    _colormap = builder.CreateString(cfg_obj['colormap'])
    _render_method = rendering_methods.get(cfg_obj['renderMethod'], None)
    _particles_count = int(cfg_obj['particlesCount'])
    _particle_ttl = int(cfg_obj['particleTTL'])
    _streamline_length = int(cfg_obj['streamlineLength'])
    _streamline_speed = cfg_obj['streamlineSpeed']
    _filter_mode = filter_modes.get(cfg_obj['filterMode'], None)

    cls.RenderingCfgStart(builder)
    cls.RenderingCfgAddRendered(builder, _rendered)
    cls.RenderingCfgAddLogscale(builder, _logscale)
    cls.RenderingCfgAddMin(builder, _min)
    cls.RenderingCfgAddMax(builder, _max)
    cls.RenderingCfgAddOpacity(builder, _opacity)
    cls.RenderingCfgAddZindex(builder, _zindex)
    cls.RenderingCfgAddColormap(builder, _colormap)
    cls.RenderingCfgAddRenderMethod(builder, _render_method)
    cls.RenderingCfgAddParticlesCount(builder, _particles_count)
    cls.RenderingCfgAddParticleTTL(builder, _particle_ttl)
    cls.RenderingCfgAddStreamlineLength(builder, _streamline_length)
    cls.RenderingCfgAddStreamlineSpeed(builder, _streamline_speed)
    cls.RenderingCfgAddFilterMode(builder, _filter_mode)

    _, _target = SEAScope.types.renderable_id.serialize(builder,
                                                        cfg_obj['target'])
    cls.RenderingCfgAddTarget(builder, _target)
    _, _color = SEAScope.types.color.serialize(builder, cfg_obj['color'])
    cls.RenderingCfgAddColor(builder, _color)
    cfg = cls.RenderingCfgEnd(builder)
    return builder, cfg


def deserialize(o):
    """"""
    cfg = {}
    cfg['rendered'] = 0 < o.Rendered()
    cfg['logscale'] = 0 < o.Logscale()
    cfg['min'] = o.Min()
    cfg['max'] = o.Max()
    cfg['opacity'] = o.Opacity()
    cfg['zindex'] = o.Zindex()
    cfg['color'] = SEAScope.types.color.deserialize(o.Color())
    cfg['colormap'] = o.Colormap()
    rendering = o.RenderMethod()
    for k, v in rendering_methods.items():
        if v == rendering:
            cfg['renderMethod'] = k
            break
    filter_mode = o.FilterMode()
    for k, v in filter_modes.items():
        if v == filter_mode:
            cfg['filterMode'] = k
            break
    cfg['particlesCount'] = o.ParticlesCount()
    cfg['particleTTL'] = o.ParticleTTL()
    cfg['streamlineLength'] = o.StreamlineLength()
    cfg['streamlineSpeed'] = o.StreamlineSpeed()
    cfg['target'] = SEAScope.types.renderable_id.deserialize(o.Target())

    return cfg
