# -*- coding: utf-8 -*-

"""
@author: <sylvain.herledan@oceandatalab.com>
@date: 2016-09-09

Copyright (C) 2016-2019 OceanDataLab

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as
published by the Free Software Foundation, either version 3 of the
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

import logging
import flatbuffers
import SEAScope.API.Shape
import SEAScope.types.gcp

logger = logging.getLogger(__name__)


def serialize(builder, shp_obj):
    """"""
    if builder is None:
        builder = flatbuffers.Builder(0)

    x_arity = int(shp_obj['xArity'])
    y_arity = int(shp_obj['yArity'])
    gcps_len = len(shp_obj['gcps'])
    SEAScope.API.Shape.ShapeStartGcpsVector(builder, gcps_len)
    for gcp in shp_obj['gcps']:
        _, _gcp = SEAScope.types.gcp.serialize(builder, gcp)
    gcps = builder.EndVector(gcps_len)

    cls = SEAScope.API.Shape
    cls.ShapeStart(builder)
    cls.ShapeAddXArity(builder, x_arity)
    cls.ShapeAddYArity(builder, y_arity)
    cls.ShapeAddGcps(builder, gcps)
    shape = cls.ShapeEnd(builder)
    return builder, shape


def deserialize(o):
    """"""
    deserializer = SEAScope.types.gcp.deserialize
    s_obj = {}
    s_obj['xArity'] = o.XArity()
    s_obj['yArity'] = o.YArity()
    gcps_count = o.GcpsLength()
    s_obj['gcps'] = [deserializer(o.Gcps(x)) for x in range(0, gcps_count)]

    return s_obj
