# -*- coding: utf-8 -*-

"""
@author: <sylvain.herledan@oceandatalab.com>
@date: 2016-09-07

Copyright (C) 2016-2019 OceanDataLab

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as
published by the Free Software Foundation, either version 3 of the
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

import logging
import flatbuffers
import SEAScope.API.Source
import SEAScope.types.tag

logger = logging.getLogger(__name__)


def serialize(builder, src_obj):
    """"""
    if builder is None:
        builder = flatbuffers.Builder(0)

    cls = SEAScope.API.Source

    _title = builder.CreateString(src_obj['title'])
    _description = builder.CreateString(src_obj['description'])

    tags = []
    tags_len = len(src_obj['tags'].keys())
    for k, v in src_obj['tags'].items():
        _, tag = SEAScope.types.tag.serialize(builder, k, v)
        tags.append(tag)
    cls.SourceStartTagsVector(builder, tags_len)
    for t in tags:
        builder.PrependUOffsetTRelative(t)
    _tags = builder.EndVector(tags_len)

    cls.SourceStart(builder)
    cls.SourceAddId(builder, int(src_obj['id']))
    cls.SourceAddTitle(builder, _title)
    cls.SourceAddDescription(builder, _description)
    cls.SourceAddTags(builder, _tags)
    source = cls.SourceEnd(builder)

    return builder, source


def deserialize(o):
    """"""
    src_obj = {}
    src_obj['id'] = o.Id()
    src_obj['title'] = o.Title()
    src_obj['description'] = o.Description()
    src_obj['tags'] = {}
    tags_count = o.TagsLength()
    for i in range(0, tags_count):
        name, value = SEAScope.types.tag.deserialize(o.Tags(i))
        src_obj['tags'][name] = value

    return src_obj
